/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.bo.queues;

import java.io.Serializable;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.hibernate.annotations.GenericGenerator;

/**
 * @author mcarpentier - EBM WebSourcing
 * 
 * A {@link QueuesStatus} is the status of a transporter's queue
 * 
 */
@Entity
public class QueuesStatus implements Serializable {

    /**
     * Serial UID
     */
    private static final long serialVersionUID = -1434628041592824361L;

    private String queuesStatusId;

    private String transporter;

    private long value;

    public QueuesStatus() {
        super();
    }

    public QueuesStatus(String queueName, long queueValue) {
        this.transporter = queueName;
        this.value = queueValue;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    @Override
    public boolean equals(Object object) {
        if (!(object instanceof QueuesStatus)) {
            return false;
        }
        final QueuesStatus rhs = (QueuesStatus) object;
        return new EqualsBuilder().appendSuper(super.equals(object)).append(this.value, rhs.value)
                .append(this.queuesStatusId, rhs.queuesStatusId).append(this.transporter,
                        rhs.transporter).isEquals();
    }

    @Id
    @GeneratedValue(generator = "system-uuid")
    @GenericGenerator(name = "system-uuid", strategy = "uuid")
    public String getQueuesStatusId() {
        return this.queuesStatusId;
    }

    public String getTransporter() {
        return this.transporter;
    }

    public long getValue() {
        return this.value;
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        return new HashCodeBuilder(-2121722667, 1142909641).appendSuper(super.hashCode()).append(
                this.value).append(this.queuesStatusId).append(this.transporter).toHashCode();
    }

    public void setQueuesStatusId(String queuesStatusId) {
        this.queuesStatusId = queuesStatusId;
    }

    public void setTransporter(String transporter) {
        this.transporter = transporter;
    }

    public void setValue(long value) {
        this.value = value;
    }
}
