/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.client.service;

import java.util.List;
import java.util.Map;

import org.ow2.petals.tools.jmx.api.PetalsJMXClient;
import org.ow2.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.ow2.petals.tools.jmx.api.exception.EndpointServiceDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.EndpointServiceServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.PerformActionErrorException;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.w3c.dom.Document;

/**
 * @author mcarpentier - EBM WebSourcing
 *
 * Deals with petals endpoints functions
 *
 */
public class EndpointService extends CollectorService {

    public EndpointService(PetalsJMXClient petalsJMXClient) {
        super(petalsJMXClient);
    }

    public List<Map<String, Object>> getAllEndpoints() throws DataCollectorClientException {

        try {

            final List<Map<String, Object>> result = this.getPetalsJMXClient()
                    .getEndpointRegistryClient().getAllEndpoints();
            return result;

        } catch (final PerformActionErrorException e) {
            throw new DataCollectorClientException("Can't retrieve all endpoints", e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final EndpointServiceDoesNotExistException e) {
            throw new DataCollectorClientException("Unable to find Monitoring Service", e);
        } catch (final EndpointServiceServiceErrorException e) {
            throw new DataCollectorClientException(e);
        }

    }

    public Document getDescription(final String serviceName, final String endpointName)
            throws DataCollectorClientException {

        try {

            final Document result = this.getPetalsJMXClient()
                    .getEndpointRegistryClient().getDescription(serviceName, endpointName);
            return result;

        } catch (final PerformActionErrorException e) {
            throw new DataCollectorClientException("Can't activate monitoring", e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final EndpointServiceDoesNotExistException e) {
            throw new DataCollectorClientException("Unable to find Monitoring Service", e);
        } catch (final EndpointServiceServiceErrorException e) {
            throw new DataCollectorClientException(e);
        }

    }

}
