/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.datacollector.client.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.hibernate.Session;
import org.ow2.petals.tools.jmx.api.PetalsJMXClient;
import org.ow2.petals.tools.jmx.api.exception.AttributeErrorException;
import org.ow2.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.ow2.petals.tools.jmx.api.exception.MonitoringDoesNotExistException;
import org.ow2.petals.tools.jmx.api.exception.MonitoringServiceErrorException;
import org.ow2.petals.tools.jmx.api.exception.PerformActionErrorException;
import org.ow2.petals.tools.webadmin.datacollector.bo.message.MessageExchangeSnapshot;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.dao.DAOException;
import org.ow2.petals.tools.webadmin.datacollector.dao.message.MessageExchangeSnapshotDAO;
import org.ow2.petals.tools.webadmin.datacollector.util.HibernateHelper;
import org.ow2.petals.tools.webadmin.datacollector.util.MessagesListener;

/**
 * @author mcarpentier - EBM WebSourcing
 * 
 * Deals with petals monitoring functions
 * 
 */
public class MonitoringService extends CollectorService {

    private MessagesListener listener;


    public MonitoringService(PetalsJMXClient petalsJMXClient) {
        super(petalsJMXClient);
    }

    public void activateMonitoring(boolean showContent) throws DataCollectorClientException {
        try {

            this.getPetalsJMXClient().getMonitoringServiceClient().activateMonitoring(
                    showContent);

        } catch (final PerformActionErrorException e) {
            throw new DataCollectorClientException("Can't activate monitoring", e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException("Can't connect to JMX monitoring service", e);
        } catch (final MonitoringDoesNotExistException e) {
            throw new DataCollectorClientException("Can't find PEtALS monitoring service", e);
        } catch (final MonitoringServiceErrorException e) {
            throw new DataCollectorClientException("Can't activate monitoring", e);
        }
    }

    public void deactivateMonitoring() throws DataCollectorClientException {
        try {

            this.getPetalsJMXClient().getMonitoringServiceClient().deactivateMonitoring();

        } catch (final PerformActionErrorException e) {
            throw new DataCollectorClientException("Can't activate monitoring", e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException("Can't connect to JMX monitoring service", e);
        } catch (final MonitoringDoesNotExistException e) {
            throw new DataCollectorClientException("Can't find PEtALS monitoring service", e);
        } catch (final MonitoringServiceErrorException e) {
            throw new DataCollectorClientException("Can't activate monitoring", e);
        }
    }

    public List<Map<String, Object>> getAllMessageExchanges() throws DataCollectorClientException {
        Session session = null;
        List<Map<String, Object>> result = new ArrayList<Map<String, Object>>();
        try {
            session = HibernateHelper.getSession();
            final List<MessageExchangeSnapshot> meList = MessageExchangeSnapshotDAO.getInstance()
                    .retrieveAll(session);
            constructMEJmxList(meList, result);
            session.close();
        } catch (final DAOException e) {
            if (session != null) {
                session.close();
            }
            throw new DataCollectorClientException("Can't get message exchanges", e);
        }
        return result;
    }

    public List<Map<String, Object>> getAllMessageExchangesSnapshots()
            throws DataCollectorClientException {
        Session session = null;
        List<Map<String, Object>> result = new ArrayList<Map<String, Object>>();
        try {
            session = HibernateHelper.getSession();
            final List<MessageExchangeSnapshot> meList = MessageExchangeSnapshotDAO.getInstance()
                    .retrieveAll(session);
            for (final MessageExchangeSnapshot mes : meList) {
                result.add(mes.generateObjectMap());
            }
            session.close();
        } catch (final DAOException e) {
            if (session != null) {
                session.close();
            }
            throw new DataCollectorClientException("Can't get message exchanges", e);
        }
        return result;
    }

    public List<Map<String, Object>> getAllMessageExchangesSnapshotsByServer(String server)
            throws DataCollectorClientException {
        Session session = null;
        List<Map<String, Object>> result = new ArrayList<Map<String, Object>>();
        try {
            session = HibernateHelper.getSession();
            final List<MessageExchangeSnapshot> meList = MessageExchangeSnapshotDAO.getInstance()
                    .retrieveByServer(session, server);
            for (final MessageExchangeSnapshot mes : meList) {
                result.add(mes.generateObjectMap());
            }
            session.close();
        } catch (final DAOException e) {
            if (session != null) {
                session.close();
            }
            throw new DataCollectorClientException("Can't get message exchanges", e);
        }
        return result;
    }

    public List<Map<String, Object>> getAllMessageExchangesSnapshotsByServerAndComponent(
            String server, String component) throws DataCollectorClientException {
        Session session = null;
        List<Map<String, Object>> result = new ArrayList<Map<String, Object>>();
        try {
            session = HibernateHelper.getSession();
            final List<MessageExchangeSnapshot> meList = MessageExchangeSnapshotDAO.getInstance()
                    .retrieveByServerAndComponent(session, server, component);
            for (final MessageExchangeSnapshot mes : meList) {
                result.add(mes.generateObjectMap());
            }
            session.close();
        } catch (final DAOException e) {
            if (session != null) {
                session.close();
            }
            throw new DataCollectorClientException("Can't get message exchanges", e);
        }
        return result;
    }

    public List<Map<String, Object>> getAllMessageExchangesSnapshotsByServerAndComponentAndEndpoint(
            String server, String component, String endpoint) throws DataCollectorClientException {
        Session session = null;
        List<Map<String, Object>> result = new ArrayList<Map<String, Object>>();
        try {
            session = HibernateHelper.getSession();
            final List<MessageExchangeSnapshot> meList = MessageExchangeSnapshotDAO.getInstance()
                    .retrieveByServerAndComponentAndEndpoint(session, server, component, endpoint);
            for (final MessageExchangeSnapshot mes : meList) {
                result.add(mes.generateObjectMap());
            }
            session.close();
        } catch (final DAOException e) {
            if (session != null) {
                session.close();
            }
            throw new DataCollectorClientException("Can't get message exchanges", e);
        }
        return result;
    }

    /**
     * Build a List of ME to pass through JMX calls
     * 
     * @param meList
     * @param result
     */
    private void constructMEJmxList(List<MessageExchangeSnapshot> meList,
            List<Map<String, Object>> result) {
        if (meList != null) {
            for (final MessageExchangeSnapshot mes : meList) {
                result.add(mes.generateObjectMap());
            }
        }
    }

    public List<Map<String, Object>> getMessageExchange(String exchangeId)
            throws DataCollectorClientException {
        Session session = null;
        List<Map<String, Object>> result = new ArrayList<Map<String, Object>>();
        try {
            session = HibernateHelper.getSession();
            final List<MessageExchangeSnapshot> meList = MessageExchangeSnapshotDAO.getInstance()
                    .retrieveByPetalsId(session, exchangeId);
            constructMEJmxList(meList, result);
            session.close();
        } catch (final DAOException e) {
            if (session != null) {
                session.close();
            }
            throw new DataCollectorClientException("Can't get message exchanges", e);
        }
        return result;
    }

    public boolean isMessageContentShown() throws DataCollectorClientException {
        try {

            return this.getPetalsJMXClient().getMonitoringServiceClient()
                    .isMessageContentShown();

        } catch (final AttributeErrorException e) {
            throw new DataCollectorClientException("Can't retrieve messageContentShown", e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException("Can't connect to JMX monitoring service", e);
        } catch (final MonitoringDoesNotExistException e) {
            throw new DataCollectorClientException("Can't find PEtALS monitoring service", e);
        } catch (final MonitoringServiceErrorException e) {
            throw new DataCollectorClientException("Can't retrieve messageContentShown", e);
        }
    }

    public boolean isMonitoring() throws DataCollectorClientException {
        try {

            return this.getPetalsJMXClient().getMonitoringServiceClient().isMonitoring();

        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException("Can't connect to JMX monitoring service", e);
        } catch (final MonitoringDoesNotExistException e) {
            throw new DataCollectorClientException("Can't find PEtALS monitoring service", e);
        } catch (final MonitoringServiceErrorException e) {
            throw new DataCollectorClientException("Can't retrieve messageContentShown", e);
        } catch (final AttributeErrorException e) {
            throw new DataCollectorClientException("Can't retrieve messageContentShown", e);
        }
    }

    public void startListening() throws DataCollectorClientException {
        try {
            if (this.listener == null) {
                this.listener = new MessagesListener();
            }
            this.getPetalsJMXClient().getMonitoringServiceClient().addListener(this.listener);
        } catch (final DataCollectorClientException e) {
            throw new DataCollectorClientException(e);
        } catch (final MonitoringServiceErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final ConnectionErrorException e) {
            throw new DataCollectorClientException(e);
        } catch (final MonitoringDoesNotExistException e) {
            throw new DataCollectorClientException("Can't get monitoring bean", e);
        }
    }
}
